@@ -3,12 +3,12 @@
  * fe-protocol3.c
  *	  functions that are specific to frontend/backend protocol version 3
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/interfaces/libpq/fe-protocol3.c,v 1.9.2.1 2003/12/28 17:44:05 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/interfaces/libpq/fe-protocol3.c,v 1.22.2.1 2005/11/22 18:23:30 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -52,7 +52,7 @@ static int	getNotify(PGconn *conn);
 static int	getCopyStart(PGconn *conn, ExecStatusType copytype);
 static int	getReadyForQuery(PGconn *conn);
 static int build_startup_packet(const PGconn *conn, char *packet,
-					 const PQEnvironmentOption * options);
+					 const PQEnvironmentOption *options);
 
 
 /*
@@ -73,8 +73,8 @@ pqParseInput3(PGconn *conn)
 	for (;;)
 	{
 		/*
-		 * Try to read a message.  First get the type code and length.
-		 * Return if not enough data.
+		 * Try to read a message.  First get the type code and length. Return
+		 * if not enough data.
 		 */
 		conn->inCursor = conn->inStart;
 		if (pqGetc(&id, conn))
@@ -83,9 +83,9 @@ pqParseInput3(PGconn *conn)
 			return;
 
 		/*
-		 * Try to validate message type/length here.  A length less than 4
-		 * is definitely broken.  Large lengths should only be believed
-		 * for a few message types.
+		 * Try to validate message type/length here.  A length less than 4 is
+		 * definitely broken.  Large lengths should only be believed for a few
+		 * message types.
 		 */
 		if (msgLength < 4)
 		{
@@ -106,20 +106,20 @@ pqParseInput3(PGconn *conn)
 		if (avail < msgLength)
 		{
 			/*
-			 * Before returning, enlarge the input buffer if needed to
-			 * hold the whole message.	This is better than leaving it to
-			 * pqReadData because we can avoid multiple cycles of
-			 * realloc() when the message is large; also, we can implement
-			 * a reasonable recovery strategy if we are unable to make the
-			 * buffer big enough.
+			 * Before returning, enlarge the input buffer if needed to hold
+			 * the whole message.  This is better than leaving it to
+			 * pqReadData because we can avoid multiple cycles of realloc()
+			 * when the message is large; also, we can implement a reasonable
+			 * recovery strategy if we are unable to make the buffer big
+			 * enough.
 			 */
 			if (pqCheckInBufferSpace(conn->inCursor + msgLength, conn))
 			{
 				/*
-				 * XXX add some better recovery code... plan is to skip
-				 * over the message using its length, then report an
-				 * error. For the moment, just treat this like loss of
-				 * sync (which indeed it might be!)
+				 * XXX add some better recovery code... plan is to skip over
+				 * the message using its length, then report an error. For the
+				 * moment, just treat this like loss of sync (which indeed it
+				 * might be!)
 				 */
 				handleSyncLoss(conn, id, msgLength);
 			}
@@ -127,20 +127,20 @@ pqParseInput3(PGconn *conn)
 		}
 
 		/*
-		 * NOTIFY and NOTICE messages can happen in any state; always
-		 * process them right away.
+		 * NOTIFY and NOTICE messages can happen in any state; always process
+		 * them right away.
 		 *
 		 * Most other messages should only be processed while in BUSY state.
-		 * (In particular, in READY state we hold off further parsing
-		 * until the application collects the current PGresult.)
+		 * (In particular, in READY state we hold off further parsing until
+		 * the application collects the current PGresult.)
 		 *
 		 * However, if the state is IDLE then we got trouble; we need to deal
 		 * with the unexpected message somehow.
 		 *
-		 * ParameterStatus ('S') messages are a special case: in IDLE state
-		 * we must process 'em (this case could happen if a new value was
-		 * adopted from config file due to SIGHUP), but otherwise we hold
-		 * off until BUSY state.
+		 * ParameterStatus ('S') messages are a special case: in IDLE state we
+		 * must process 'em (this case could happen if a new value was adopted
+		 * from config file due to SIGHUP), but otherwise we hold off until
+		 * BUSY state.
 		 */
 		if (id == 'A')
 		{
@@ -163,9 +163,9 @@ pqParseInput3(PGconn *conn)
 			 * ERROR messages are displayed using the notice processor;
 			 * ParameterStatus is handled normally; anything else is just
 			 * dropped on the floor after displaying a suitable warning
-			 * notice.	(An ERROR is very possibly the backend telling us
-			 * why it is about to close the connection, so we don't want
-			 * to just discard it...)
+			 * notice.	(An ERROR is very possibly the backend telling us why
+			 * it is about to close the connection, so we don't want to just
+			 * discard it...)
 			 */
 			if (id == 'E')
 			{
@@ -180,7 +180,7 @@ pqParseInput3(PGconn *conn)
 			else
 			{
 				pqInternalNotice(&conn->noticeHooks,
-					"message type 0x%02x arrived from server while idle",
+						"message type 0x%02x arrived from server while idle",
 								 id);
 				/* Discard the unexpected message */
 				conn->inCursor += msgLength;
@@ -197,8 +197,12 @@ pqParseInput3(PGconn *conn)
 					if (pqGets(&conn->workBuffer, conn))
 						return;
 					if (conn->result == NULL)
+					{
 						conn->result = PQmakeEmptyPGresult(conn,
-													   PGRES_COMMAND_OK);
+														   PGRES_COMMAND_OK);
+						if (!conn->result)
+							return;
+					}
 					strncpy(conn->result->cmdStatus, conn->workBuffer.data,
 							CMDSTATUS_LEN);
 					conn->asyncStatus = PGASYNC_READY;
@@ -215,11 +219,28 @@ pqParseInput3(PGconn *conn)
 					break;
 				case 'I':		/* empty query */
 					if (conn->result == NULL)
+					{
 						conn->result = PQmakeEmptyPGresult(conn,
-													  PGRES_EMPTY_QUERY);
+														   PGRES_EMPTY_QUERY);
+						if (!conn->result)
+							return;
+					}
 					conn->asyncStatus = PGASYNC_READY;
 					break;
 				case '1':		/* Parse Complete */
+					/* If we're doing PQprepare, we're done; else ignore */
+					if (conn->queryclass == PGQUERY_PREPARE)
+					{
+						if (conn->result == NULL)
+						{
+							conn->result = PQmakeEmptyPGresult(conn,
+														   PGRES_COMMAND_OK);
+							if (!conn->result)
+								return;
+						}
+						conn->asyncStatus = PGASYNC_READY;
+					}
+					break;
 				case '2':		/* Bind Complete */
 				case '3':		/* Close Complete */
 					/* Nothing to do for these message types */
@@ -231,9 +252,9 @@ pqParseInput3(PGconn *conn)
 				case 'K':		/* secret key data from the backend */
 
 					/*
-					 * This is expected only during backend startup, but
-					 * it's just as easy to handle it as part of the main
-					 * loop.  Save the data and continue processing.
+					 * This is expected only during backend startup, but it's
+					 * just as easy to handle it as part of the main loop.
+					 * Save the data and continue processing.
 					 */
 					if (pqGetInt(&(conn->be_pid), 4, conn))
 						return;
@@ -251,21 +272,22 @@ pqParseInput3(PGconn *conn)
 					{
 						/*
 						 * A new 'T' message is treated as the start of
-						 * another PGresult.  (It is not clear that this
-						 * is really possible with the current backend.)
-						 * We stop parsing until the application accepts
-						 * the current result.
+						 * another PGresult.  (It is not clear that this is
+						 * really possible with the current backend.) We stop
+						 * parsing until the application accepts the current
+						 * result.
 						 */
 						conn->asyncStatus = PGASYNC_READY;
 						return;
 					}
 					break;
 				case 'n':		/* No Data */
+
 					/*
 					 * NoData indicates that we will not be seeing a
-					 * RowDescription message because the statement or
-					 * portal inquired about doesn't return rows.
-					 * Set up a COMMAND_OK result, instead of TUPLES_OK.
+					 * RowDescription message because the statement or portal
+					 * inquired about doesn't return rows. Set up a COMMAND_OK
+					 * result, instead of TUPLES_OK.
 					 */
 					if (conn->result == NULL)
 						conn->result = PQmakeEmptyPGresult(conn,
@@ -280,12 +302,11 @@ pqParseInput3(PGconn *conn)
 							return;
 					}
 					else if (conn->result != NULL &&
-						 conn->result->resultStatus == PGRES_FATAL_ERROR)
+							 conn->result->resultStatus == PGRES_FATAL_ERROR)
 					{
 						/*
-						 * We've already choked for some reason.  Just
-						 * discard tuples till we get to the end of the
-						 * query.
+						 * We've already choked for some reason.  Just discard
+						 * tuples till we get to the end of the query.
 						 */
 						conn->inCursor += msgLength;
 					}
@@ -313,19 +334,19 @@ pqParseInput3(PGconn *conn)
 				case 'd':		/* Copy Data */
 
 					/*
-					 * If we see Copy Data, just silently drop it.	This
-					 * would only occur if application exits COPY OUT mode
-					 * too early.
+					 * If we see Copy Data, just silently drop it.	This would
+					 * only occur if application exits COPY OUT mode too
+					 * early.
 					 */
 					conn->inCursor += msgLength;
 					break;
 				case 'c':		/* Copy Done */
 
 					/*
-					 * If we see Copy Done, just silently drop it.	This
-					 * is the normal case during PQendcopy.  We will keep
-					 * swallowing data, expecting to see command-complete
-					 * for the COPY command.
+					 * If we see Copy Done, just silently drop it.	This is
+					 * the normal case during PQendcopy.  We will keep
+					 * swallowing data, expecting to see command-complete for
+					 * the COPY command.
 					 */
 					break;
 				default:
@@ -352,7 +373,7 @@ pqParseInput3(PGconn *conn)
 		{
 			/* Trouble --- report it */
 			printfPQExpBuffer(&conn->errorMessage,
-							  libpq_gettext("Message contents do not agree with length in message type \"%c\"\n"),
+							  libpq_gettext("message contents do not agree with length in message type \"%c\"\n"),
 							  id);
 			/* build an error result holding the error message */
 			pqSaveErrorResult(conn);
@@ -373,7 +394,7 @@ handleSyncLoss(PGconn *conn, char id, int msgLength)
 {
 	printfPQExpBuffer(&conn->errorMessage,
 					  libpq_gettext(
-									"lost synchronization with server: got message type \"%c\", length %d\n"),
+	"lost synchronization with server: got message type \"%c\", length %d\n"),
 					  id, msgLength);
 	/* build an error result holding the error message */
 	pqSaveErrorResult(conn);
@@ -402,14 +423,13 @@ getRowDescriptions(PGconn *conn)
 	int			i;
 
 	result = PQmakeEmptyPGresult(conn, PGRES_TUPLES_OK);
+	if (!result)
+		goto failure;
 
 	/* parseInput already read the 'T' label and message length. */
 	/* the next two bytes are the number of fields	*/
 	if (pqGetInt(&(result->numAttributes), 2, conn))
-	{
-		PQclear(result);
-		return EOF;
-	}
+		goto failure;
 	nfields = result->numAttributes;
 
 	/* allocate space for the attribute descriptors */
@@ -417,7 +437,9 @@ getRowDescriptions(PGconn *conn)
 	{
 		result->attDescs = (PGresAttDesc *)
 			pqResultAlloc(result, nfields * sizeof(PGresAttDesc), TRUE);
-		MemSet((char *) result->attDescs, 0, nfields * sizeof(PGresAttDesc));
+		if (!result->attDescs)
+			goto failure;
+		MemSet(result->attDescs, 0, nfields * sizeof(PGresAttDesc));
 	}
 
 	/* result->binary is true only if ALL columns are binary */
@@ -441,8 +463,7 @@ getRowDescriptions(PGconn *conn)
 			pqGetInt(&atttypmod, 4, conn) ||
 			pqGetInt(&format, 2, conn))
 		{
-			PQclear(result);
-			return EOF;
+			goto failure;
 		}
 
 		/*
@@ -455,6 +476,8 @@ getRowDescriptions(PGconn *conn)
 
 		result->attDescs[i].name = pqResultStrdup(result,
 												  conn->workBuffer.data);
+		if (!result->attDescs[i].name)
+			goto failure;
 		result->attDescs[i].tableid = tableid;
 		result->attDescs[i].columnid = columnid;
 		result->attDescs[i].format = format;
@@ -469,6 +492,10 @@ getRowDescriptions(PGconn *conn)
 	/* Success! */
 	conn->result = result;
 	return 0;
+
+failure:
+	PQclear(result);
+	return EOF;
 }
 
 /*
@@ -497,7 +524,7 @@ getAnotherTuple(PGconn *conn, int msgLength)
 			pqResultAlloc(result, nfields * sizeof(PGresAttValue), TRUE);
 		if (conn->curTuple == NULL)
 			goto outOfMemory;
-		MemSet((char *) conn->curTuple, 0, nfields * sizeof(PGresAttValue));
+		MemSet(conn->curTuple, 0, nfields * sizeof(PGresAttValue));
 	}
 	tup = conn->curTuple;
 
@@ -509,7 +536,7 @@ getAnotherTuple(PGconn *conn, int msgLength)
 	{
 		/* Replace partially constructed result with an error result */
 		printfPQExpBuffer(&conn->errorMessage,
-				 libpq_gettext("unexpected field count in D message\n"));
+				 libpq_gettext("unexpected field count in \"D\" message\n"));
 		pqSaveErrorResult(conn);
 		/* Discard the failed message by pretending we read it */
 		conn->inCursor = conn->inStart + 5 + msgLength;
@@ -583,27 +610,29 @@ outOfMemory:
 int
 pqGetErrorNotice3(PGconn *conn, bool isError)
 {
-	PGresult   *res;
+	PGresult   *res = NULL;
 	PQExpBufferData workBuf;
 	char		id;
 	const char *val;
 
-	/*
-	 * Make a PGresult to hold the accumulated fields.	We temporarily lie
-	 * about the result status, so that PQmakeEmptyPGresult doesn't
-	 * uselessly copy conn->errorMessage.
-	 */
-	res = PQmakeEmptyPGresult(conn, PGRES_EMPTY_QUERY);
-	res->resultStatus = isError ? PGRES_FATAL_ERROR : PGRES_NONFATAL_ERROR;
-
 	/*
 	 * Since the fields might be pretty long, we create a temporary
-	 * PQExpBuffer rather than using conn->workBuffer.	workBuffer is
-	 * intended for stuff that is expected to be short.  We shouldn't use
+	 * PQExpBuffer rather than using conn->workBuffer.	workBuffer is intended
+	 * for stuff that is expected to be short.	We shouldn't use
 	 * conn->errorMessage either, since this might be only a notice.
 	 */
 	initPQExpBuffer(&workBuf);
 
+	/*
+	 * Make a PGresult to hold the accumulated fields.	We temporarily lie
+	 * about the result status, so that PQmakeEmptyPGresult doesn't uselessly
+	 * copy conn->errorMessage.
+	 */
+	res = PQmakeEmptyPGresult(conn, PGRES_EMPTY_QUERY);
+	if (!res)
+		goto fail;
+	res->resultStatus = isError ? PGRES_FATAL_ERROR : PGRES_NONFATAL_ERROR;
+
 	/*
 	 * Read the fields and save into res.
 	 */
@@ -640,6 +669,16 @@ pqGetErrorNotice3(PGconn *conn, bool isError)
 		/* translator: %s represents a digit string */
 		appendPQExpBuffer(&workBuf, libpq_gettext(" at character %s"), val);
 	}
+	else
+	{
+		val = PQresultErrorField(res, PG_DIAG_INTERNAL_POSITION);
+		if (val)
+		{
+			/* translator: %s represents a digit string */
+			appendPQExpBuffer(&workBuf, libpq_gettext(" at character %s"),
+							  val);
+		}
+	}
 	appendPQExpBufferChar(&workBuf, '\n');
 	if (conn->verbosity != PQERRORS_TERSE)
 	{
@@ -649,6 +688,9 @@ pqGetErrorNotice3(PGconn *conn, bool isError)
 		val = PQresultErrorField(res, PG_DIAG_MESSAGE_HINT);
 		if (val)
 			appendPQExpBuffer(&workBuf, libpq_gettext("HINT:  %s\n"), val);
+		val = PQresultErrorField(res, PG_DIAG_INTERNAL_QUERY);
+		if (val)
+			appendPQExpBuffer(&workBuf, libpq_gettext("QUERY:  %s\n"), val);
 		val = PQresultErrorField(res, PG_DIAG_CONTEXT);
 		if (val)
 			appendPQExpBuffer(&workBuf, libpq_gettext("CONTEXT:  %s\n"), val);
@@ -679,6 +721,8 @@ pqGetErrorNotice3(PGconn *conn, bool isError)
 	if (isError)
 	{
 		res->errMsg = pqResultStrdup(res, workBuf.data);
+		if (!res->errMsg)
+			goto fail;
 		pqClearAsyncResult(conn);
 		conn->result = res;
 		resetPQExpBuffer(&conn->errorMessage);
@@ -763,9 +807,9 @@ getNotify(PGconn *conn)
 	}
 
 	/*
-	 * Store the strings right after the PQnotify structure so it can all
-	 * be freed at once.  We don't use NAMEDATALEN because we don't want
-	 * to tie this interface to a specific server name length.
+	 * Store the strings right after the PQnotify structure so it can all be
+	 * freed at once.  We don't use NAMEDATALEN because we don't want to tie
+	 * this interface to a specific server name length.
 	 */
 	nmlen = strlen(svname);
 	extralen = strlen(conn->workBuffer.data);
@@ -777,7 +821,12 @@ getNotify(PGconn *conn)
 		newNotify->extra = newNotify->relname + nmlen + 1;
 		strcpy(newNotify->extra, conn->workBuffer.data);
 		newNotify->be_pid = be_pid;
-		DLAddTail(conn->notifyList, DLNewElem(newNotify));
+		newNotify->next = NULL;
+		if (conn->notifyTail)
+			conn->notifyTail->next = newNotify;
+		else
+			conn->notifyHead = newNotify;
+		conn->notifyTail = newNotify;
 	}
 
 	free(svname);
@@ -797,19 +846,15 @@ getCopyStart(PGconn *conn, ExecStatusType copytype)
 	int			i;
 
 	result = PQmakeEmptyPGresult(conn, copytype);
+	if (!result)
+		goto failure;
 
 	if (pqGetc(&conn->copy_is_binary, conn))
-	{
-		PQclear(result);
-		return EOF;
-	}
+		goto failure;
 	result->binary = conn->copy_is_binary;
 	/* the next two bytes are the number of fields	*/
 	if (pqGetInt(&(result->numAttributes), 2, conn))
-	{
-		PQclear(result);
-		return EOF;
-	}
+		goto failure;
 	nfields = result->numAttributes;
 
 	/* allocate space for the attribute descriptors */
@@ -817,7 +862,9 @@ getCopyStart(PGconn *conn, ExecStatusType copytype)
 	{
 		result->attDescs = (PGresAttDesc *)
 			pqResultAlloc(result, nfields * sizeof(PGresAttDesc), TRUE);
-		MemSet((char *) result->attDescs, 0, nfields * sizeof(PGresAttDesc));
+		if (!result->attDescs)
+			goto failure;
+		MemSet(result->attDescs, 0, nfields * sizeof(PGresAttDesc));
 	}
 
 	for (i = 0; i < nfields; i++)
@@ -825,23 +872,23 @@ getCopyStart(PGconn *conn, ExecStatusType copytype)
 		int			format;
 
 		if (pqGetInt(&format, 2, conn))
-		{
-			PQclear(result);
-			return EOF;
-		}
+			goto failure;
 
 		/*
 		 * Since pqGetInt treats 2-byte integers as unsigned, we need to
 		 * coerce these results to signed form.
 		 */
 		format = (int) ((int16) format);
-
 		result->attDescs[i].format = format;
 	}
 
 	/* Success! */
 	conn->result = result;
 	return 0;
+
+failure:
+	PQclear(result);
+	return EOF;
 }
 
 /*
@@ -892,9 +939,9 @@ pqGetCopyData3(PGconn *conn, char **buffer, int async)
 	for (;;)
 	{
 		/*
-		 * Do we have the next input message?  To make life simpler for
-		 * async callers, we keep returning 0 until the next message is
-		 * fully available, even if it is not Copy Data.
+		 * Do we have the next input message?  To make life simpler for async
+		 * callers, we keep returning 0 until the next message is fully
+		 * available, even if it is not Copy Data.
 		 */
 		conn->inCursor = conn->inStart;
 		if (pqGetc(&id, conn))
@@ -969,7 +1016,7 @@ pqGetline3(PGconn *conn, char *s, int maxlen)
 		conn->copy_is_binary)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-				  libpq_gettext("PQgetline: not doing text COPY OUT\n"));
+					  libpq_gettext("PQgetline: not doing text COPY OUT\n"));
 		*s = '\0';
 		return EOF;
 	}
@@ -1022,9 +1069,8 @@ pqGetlineAsync3(PGconn *conn, char *buffer, int bufsize)
 
 	/*
 	 * Recognize the next input message.  To make life simpler for async
-	 * callers, we keep returning 0 until the next message is fully
-	 * available even if it is not Copy Data.  This should keep PQendcopy
-	 * from blocking.
+	 * callers, we keep returning 0 until the next message is fully available
+	 * even if it is not Copy Data.  This should keep PQendcopy from blocking.
 	 */
 	conn->inCursor = conn->inStart;
 	if (pqGetc(&id, conn))
@@ -1036,8 +1082,8 @@ pqGetlineAsync3(PGconn *conn, char *buffer, int bufsize)
 		return 0;
 
 	/*
-	 * Cannot proceed unless it's a Copy Data message.  Anything else
-	 * means end of copy mode.
+	 * Cannot proceed unless it's a Copy Data message.  Anything else means
+	 * end of copy mode.
 	 */
 	if (id != 'd')
 		return -1;
@@ -1094,11 +1140,12 @@ pqEndcopy3(PGconn *conn)
 		if (pqPutMsgStart('c', false, conn) < 0 ||
 			pqPutMsgEnd(conn) < 0)
 			return 1;
+
 		/*
-		 * If we sent the COPY command in extended-query mode, we must
-		 * issue a Sync as well.
+		 * If we sent the COPY command in extended-query mode, we must issue a
+		 * Sync as well.
 		 */
-		if (conn->ext_query)
+		if (conn->queryclass != PGQUERY_SIMPLE)
 		{
 			if (pqPutMsgStart('S', false, conn) < 0 ||
 				pqPutMsgEnd(conn) < 0)
@@ -1107,8 +1154,8 @@ pqEndcopy3(PGconn *conn)
 	}
 
 	/*
-	 * make sure no data is waiting to be sent, abort if we are
-	 * non-blocking and the flush fails
+	 * make sure no data is waiting to be sent, abort if we are non-blocking
+	 * and the flush fails
 	 */
 	if (pqFlush(conn) && pqIsnonblocking(conn))
 		return (1);
@@ -1118,12 +1165,11 @@ pqEndcopy3(PGconn *conn)
 	resetPQExpBuffer(&conn->errorMessage);
 
 	/*
-	 * Non blocking connections may have to abort at this point.  If
-	 * everyone played the game there should be no problem, but in error
-	 * scenarios the expected messages may not have arrived yet.  (We are
-	 * assuming that the backend's packetizing will ensure that
-	 * CommandComplete arrives along with the CopyDone; are there corner
-	 * cases where that doesn't happen?)
+	 * Non blocking connections may have to abort at this point.  If everyone
+	 * played the game there should be no problem, but in error scenarios the
+	 * expected messages may not have arrived yet.	(We are assuming that the
+	 * backend's packetizing will ensure that CommandComplete arrives along
+	 * with the CopyDone; are there corner cases where that doesn't happen?)
 	 */
 	if (pqIsnonblocking(conn) && PQisBusy(conn))
 		return (1);
@@ -1142,8 +1188,8 @@ pqEndcopy3(PGconn *conn)
 	 * Trouble. For backwards-compatibility reasons, we issue the error
 	 * message as if it were a notice (would be nice to get rid of this
 	 * silliness, but too many apps probably don't handle errors from
-	 * PQendcopy reasonably).  Note that the app can still obtain the
-	 * error status from the PGconn object.
+	 * PQendcopy reasonably).  Note that the app can still obtain the error
+	 * status from the PGconn object.
 	 */
 	if (conn->errorMessage.len > 0)
 	{
@@ -1244,8 +1290,7 @@ pqFunctionCall3(PGconn *conn, Oid fnid,
 		}
 
 		/*
-		 * Scan the message. If we run out of data, loop around to try
-		 * again.
+		 * Scan the message. If we run out of data, loop around to try again.
 		 */
 		needInput = true;
 
@@ -1256,9 +1301,9 @@ pqFunctionCall3(PGconn *conn, Oid fnid,
 			continue;
 
 		/*
-		 * Try to validate message type/length here.  A length less than 4
-		 * is definitely broken.  Large lengths should only be believed
-		 * for a few message types.
+		 * Try to validate message type/length here.  A length less than 4 is
+		 * definitely broken.  Large lengths should only be believed for a few
+		 * message types.
 		 */
 		if (msgLength < 4)
 		{
@@ -1279,16 +1324,16 @@ pqFunctionCall3(PGconn *conn, Oid fnid,
 		if (avail < msgLength)
 		{
 			/*
-			 * Before looping, enlarge the input buffer if needed to hold
-			 * the whole message.  See notes in parseInput.
+			 * Before looping, enlarge the input buffer if needed to hold the
+			 * whole message.  See notes in parseInput.
 			 */
 			if (pqCheckInBufferSpace(conn->inCursor + msgLength, conn))
 			{
 				/*
-				 * XXX add some better recovery code... plan is to skip
-				 * over the message using its length, then report an
-				 * error. For the moment, just treat this like loss of
-				 * sync (which indeed it might be!)
+				 * XXX add some better recovery code... plan is to skip over
+				 * the message using its length, then report an error. For the
+				 * moment, just treat this like loss of sync (which indeed it
+				 * might be!)
 				 */
 				handleSyncLoss(conn, id, msgLength);
 				break;
@@ -1298,8 +1343,8 @@ pqFunctionCall3(PGconn *conn, Oid fnid,
 
 		/*
 		 * We should see V or E response to the command, but might get N
-		 * and/or A notices first. We also need to swallow the final Z
-		 * before returning.
+		 * and/or A notices first. We also need to swallow the final Z before
+		 * returning.
 		 */
 		switch (id)
 		{
@@ -1355,7 +1400,7 @@ pqFunctionCall3(PGconn *conn, Oid fnid,
 			default:
 				/* The backend violates the protocol. */
 				printfPQExpBuffer(&conn->errorMessage,
-							  libpq_gettext("protocol error: id=0x%x\n"),
+								  libpq_gettext("protocol error: id=0x%x\n"),
 								  id);
 				pqSaveErrorResult(conn);
 				/* trust the specified message length as what to skip */
@@ -1385,7 +1430,7 @@ pqFunctionCall3(PGconn *conn, Oid fnid,
  */
 char *
 pqBuildStartupPacket3(PGconn *conn, int *packetlen,
-					  const PQEnvironmentOption * options)
+					  const PQEnvironmentOption *options)
 {
 	char	   *startpacket;
 
@@ -1408,7 +1453,7 @@ pqBuildStartupPacket3(PGconn *conn, int *packetlen,
  */
 static int
 build_startup_packet(const PGconn *conn, char *packet,
-					 const PQEnvironmentOption * options)
+					 const PQEnvironmentOption *options)
 {
 	int			packet_len = 0;
 	const PQEnvironmentOption *next_eo;
@@ -1458,7 +1503,7 @@ build_startup_packet(const PGconn *conn, char *packet,
 
 		if ((val = getenv(next_eo->envName)) != NULL)
 		{
-			if (strcasecmp(val, "default") != 0)
+			if (pg_strcasecmp(val, "default") != 0)
 			{
 				if (packet)
 					strcpy(packet + packet_len, next_eo->pgName);
